@@ -162,11 +162,14 @@ apiKeys = {
         "K84DVIXX16BZEPR4J6NXE9HHVNTSFAQDIC",
         "NNJ96AD95RXNUN7C4QJ34D53CHGMUU1J5H"
     ],
-    "moonriver": ["E9W9NPRB1NCJTEAAA512U8UJYRD55MZSH3", "Y7MNACP4TTIC7FH962FTHQD3QJYJZ7USU4", "IWZMJSBHN17PGCFHXY1EFTGVXZUSIF7FUC"],
-    "moonbeam": ["PPGYVB7AR97IDSVKZZJ7TD67WVNDBM74AY", "IV47Y5R8SRKW9KC5DVDYB5RVC3GCV5IHKA", "MQPRQUKMHEFRE7KK6WFXIHZTDMC8RCWX65"],
-    "moonbase": ["","",""],
-    "boba": ["8KNECUPEU3FW673BA65P5MD13WRHHWVWTX","UMBCJRG9TNMB4UUPF4CC679AAPZYRXRRH6","UYDGY2WRN29IG27YSVJ5NNQ3792VZQAH2A"],
-    "boba-testnet": ["","",""],
+    "moonriver": ["E9W9NPRB1NCJTEAAA512U8UJYRD55MZSH3", "Y7MNACP4TTIC7FH962FTHQD3QJYJZ7USU4",
+                  "IWZMJSBHN17PGCFHXY1EFTGVXZUSIF7FUC"],
+    "moonbeam": ["PPGYVB7AR97IDSVKZZJ7TD67WVNDBM74AY", "IV47Y5R8SRKW9KC5DVDYB5RVC3GCV5IHKA",
+                 "MQPRQUKMHEFRE7KK6WFXIHZTDMC8RCWX65"],
+    "moonbase": ["", "", ""],
+    "boba": ["8KNECUPEU3FW673BA65P5MD13WRHHWVWTX", "UMBCJRG9TNMB4UUPF4CC679AAPZYRXRRH6",
+             "UYDGY2WRN29IG27YSVJ5NNQ3792VZQAH2A"],
+    "boba-testnet": ["", "", ""],
 }
 
 defaultFolder = "contract"
@@ -193,21 +196,79 @@ reqUrl = {
     "moonriver": "https://api-moonriver.moonscan.io/api?module=contract&action=getsourcecode&address=",
     "moonbeam": "https://api-moonbeam.moonscan.io/api?module=contract&action=getsourcecode&address=",
     "moonbase": "https://api-moonbase.moonscan.io/api?module=contract&action=getsourcecode&address=",
-    "boba":"https://api.bobascan.com/api?module=contract&action=getsourcecode&address=",
-    "boba-testnet":"https://api-testnet.bobascan.com/api?module=contract&action=getsourcecode&address="
+    "boba": "https://api.bobascan.com/api?module=contract&action=getsourcecode&address=",
+    "boba-testnet": "https://api-testnet.bobascan.com/api?module=contract&action=getsourcecode&address="
 
 }
 
+txhashConfig = {
+    'eth': {
+        "url": "https://etherscan.io/vmtrace?txhash={}&type=parity#raw",
+        "re": r"<preid='editor'>(.*?)</pre>"
+    },
+    'bsc': {
+        "url": "https://www.bscscan.com/vmtrace?txhash={}&type=gethtrace2",
+        "re": r"<preid='editor'>(.*?)</pre>"
+    },
+    'poly': {
+        "url": "",
+        "re": ""
+    },
+    'heco': {
+        "url": "https://www.hecoinfo.com/api/v1/chain/txs/detail?txHash={}&chainId=HECO",
+        "re": ""
+    },
+    'fantom': {
+        "url": "",
+        "re": ""
+    },
+    'avax': {
+        "url": "",
+        "re": ""
+    },
+    'arbi': {
+        "url": "https://arbiscan.io/vmtrace?txhash={}&type=gethtrace2",
+        "re": r"<preid='editor'>(.*?)</pre>"
+    },
+    'tron': {
+        "url": "",
+        "re": ""
+    },
+    'cronos': {
+        "url": "",
+        "re": ""
+    },
+    'okex': {
+        "url": "",
+        "re": ""
+    },
+    'boba': {
+        "url": "https://bobascan.com/vmtrace?txhash={}&type=gethtrace2",
+        "re": r"<preid='editor'>(.*?)</pre>"
+    },
+    'moonbeam': {
+        "url": "",
+        "re": ""
+    },
+    'moonriver': {
+        "url": "",
+        "re": ""
+    }
+}
+
 currentPath = os.getcwd()
 
 proxy = ""
 
 contractIndex = 0
 
+contractInfo = {}
+
+proxyContract = {}
 
 def _argparse():
     parser = argparse.ArgumentParser(
-        description="To get contract source code. \n\nSupport network: \nHeco|ETH|BSC|Fantom|Poly|AVAX|ARBI|Tron\nCronos|moonbeam|moonriver|boba|okex\navax-testnet|arbi-testnet|poly-testnet\nbsc-testnet|heco-testnet|ftm-testnet\nGoerli|Kovan|Rinkeby|Ropsten\nmoonbase|boba-testnet",
+        description="To get contract source code. \n\nSupport network: \nHeco|ETH|BSC|Fantom|Poly|AVAX|ARBI|Tron\nCronos|moonbeam|moonriver|boba|okex\navax-testnet|arbi-testnet|poly-testnet\nbsc-testnet|heco-testnet|ftm-testnet\nGoerli|Kovan|Rinkeby|Ropsten\nmoonbase|boba-testnet\n\nGet code by tx only supports:\nBSC|ETH|BOBA|ARBI|HECO",
         formatter_class=RawTextHelpFormatter)
     parser.add_argument('-i', default='', dest='inputFile', help='Input file path including contract addresses.')
     parser.add_argument('-o', default='', dest='outputFolder', help='Choose a folder to export.')
@@ -215,6 +276,8 @@ def _argparse():
     parser.add_argument('-n', default='', dest='network', help='Which network to get source code.')
     parser.add_argument('-k', action="store_true", dest='key', help='Provide some api keys.')
     parser.add_argument('-p', default='', dest='proxy', help='Use a proxy.')
+    parser.add_argument('-t', default='', dest='txhash',
+                        help='Get the relevant contract source code in the specified transaction.')
 
     return parser.parse_args()
 
@@ -299,7 +362,7 @@ def getCode(inputFile, outputFolder, address, network):
                 print("Erorr input")
                 os._exit(0)
             if addresses != []:
-                sendRequest(addresses, outputFolder, network)
+                sendRequest(addresses, outputFolder, network,"")
             else:
                 print("Erorr address")
                 os._exit(0)
@@ -311,6 +374,17 @@ def getCode(inputFile, outputFolder, address, network):
         os._exit(0)
 
 
+def saveInfo(address, contractName):
+    try:
+        if address not in contractInfo.keys():
+            contractInfo[address] = contractName
+    except Exception as e:
+        print("--------------------------------------")
+        print("error line:", e.__traceback__.tb_lineno)
+        print("error type:", e)
+        print("--------------------------------------")
+
+
 def getAddresses(inputFile, isTron, addString):
     try:
         if inputFile != "":
@@ -339,7 +413,7 @@ def getAddresses(inputFile, isTron, addString):
         os._exit(0)
 
 
-def exportResult(result, outputFolder, network):
+def exportResult(result, outputFolder, network, address,typeOf):
     global contractIndex
     if "ContractName" not in result.keys():
         return
@@ -347,6 +421,7 @@ def exportResult(result, outputFolder, network):
         index = 0
         isMultiFile = False
         contractName = result['ContractName']
+        saveInfo(address, contractName)
         sourceCode = result['SourceCode'].replace("\r\n", "\n")
         if sourceCode.find('.sol":{"content":') != -1:
             isMultiFile = True
@@ -369,6 +444,8 @@ def exportResult(result, outputFolder, network):
                 print(f'{contractIndex}: {contractFolder + "/" + contractName}'.replace('//', '/'))
                 contractIndex += 1
         else:
+            if contractName == "":
+                return
             contractFolder = defaultFolder if outputFolder == "" else outputFolder
             mkdir(contractFolder)
             contractName = contractName + ".sol"
@@ -382,13 +459,19 @@ def exportResult(result, outputFolder, network):
             contractIndex += 1
         if network == "cronos":
             if result["IsProxy"] == True:
+                if typeOf == "bytx":
+                    proxyContract[address]=result['ImplementationAddress']
+                    return
                 addresses = []
                 addresses.append(result['ImplementationAddress'])
-                sendRequest(addresses, "ImplementationAddress", network)
+                sendRequest(addresses, "ImplementationAddress", network,"")
         elif result['Implementation'] != "":
+            if typeOf == "bytx":
+                proxyContract[address] = result['Implementation']
+                return
             addresses = []
             addresses.append(result['Implementation'])
-            sendRequest(addresses, "Implementation", network)
+            sendRequest(addresses, "Implementation", network,"")
     except Exception as e:
         print("--------------------------------------")
         print("error line:", e.__traceback__.tb_lineno)
@@ -408,7 +491,7 @@ def mkdir(path):
         return False
 
 
-def sendRequest(addresses, outputFolder, network):
+def sendRequest(addresses, outputFolder, network,typeOf):
     try:
         proxies = {
             "https": "http://" + proxy,
@@ -430,7 +513,7 @@ def sendRequest(addresses, outputFolder, network):
                 outputData = results
             elif type(results) == list:
                 outputData = results[0]
-            exportResult(outputData, outputFolder, network)
+            exportResult(outputData, outputFolder, network, address,typeOf)
     except Exception as e:
         print("--------------------------------------")
         print("error line:", e.__traceback__.tb_lineno)
@@ -462,7 +545,7 @@ def sendTron(addresses, outputFolder):
                     mkdir(os.path.split(contractFolder + "//" + contractName)[0])
                     with open(contractFolder + "//" + contractName, "w+", encoding='utf-8') as fw:
                         fw.write(str(base64.b64decode(code['code']), 'utf-8').replace('\r\n', '\n'))
-                    print(f'{contractIndex}: {contractFolder + "/" + contractName}'.replace('//', '/'))
+                    print(f'{contractIndex}: {contractFolder + "/" + contractName}\t{address}'.replace('//', '/'))
                     contractIndex += 1
     except Exception as e:
         print("--------------------------------------")
@@ -484,8 +567,8 @@ def sendOkex(addresses, outputFolder):
                     contract_code_list = json.loads(req.text)["data"]["contractSourceList"]
                 else:
                     contract_code_list = [{
-                        "name":json.loads(req.text)["data"]["name"],
-                        "sourceCode":json.loads(req.text)["data"]["contractSource"],
+                        "name": json.loads(req.text)["data"]["name"],
+                        "sourceCode": json.loads(req.text)["data"]["contractSource"],
                     }]
                 main_contract = json.loads(req.text)["data"]["name"]
                 contractFolder = defaultFolder if outputFolder == "" else outputFolder
@@ -515,6 +598,51 @@ def sendOkex(addresses, outputFolder):
         os._exit(0)
 
 
+def getAddress(jsonData):
+    try:
+        addressList = list(
+            set(re.findall(r'[\'\"]{1}[\s]{0,5}:[\s]{0,5}[\'\"]{1}(0x[a-zA-Z0-9)]{40})[\'\"]{1}', jsonData)))
+        return addressList
+    except Exception as e:
+        print("--------------------------------------")
+        print("error line:", e.__traceback__.tb_lineno)
+        print("error type:", e)
+        print("--------------------------------------")
+
+
+def traceTransaction(txhash, proxy, network, outputFolder):
+    try:
+        url = txhashConfig[network]['url'].format(txhash)
+        if url == "":
+            print(f"{network} is not supported (at least for now).")
+            return []
+        header = {
+            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/107.0.0.0 Safari/537.36"
+        }
+        if proxy != "":
+            proxies = {
+                "https": "http://" + proxy,
+                "http": "http://" + proxy
+            }
+            req = requests.get(url, headers=header, proxies=proxies)
+        else:
+            req = requests.get(url, headers=header)
+        if txhashConfig[network]['re'] != "":
+            data = req.text.replace("\r\n", "").replace(' ', '').replace('\n', '')
+            result = re.findall(txhashConfig[network]['re'], data)[0]
+        else:
+            result = "{}".format(req.json()["data"]["overview"]["internalTxns"])
+        if network == 'eth':
+            result = "[" + result + "]"
+        addressList = getAddress(result)
+        sendRequest(addressList, outputFolder, network, "bytx")
+    except Exception as e:
+        print("--------------------------------------")
+        print("error line:", e.__traceback__.tb_lineno)
+        print("error type:", e)
+        print("--------------------------------------")
+
+
 def main():
     try:
         parser = _argparse()
@@ -522,22 +650,33 @@ def main():
         outputFolder = parser.outputFolder
         address = parser.address
         network = parser.network.lower()
-        # print(parser)
+        txhash = parser.txhash
         global proxy
         proxy = parser.proxy
         key = parser.key
         if key:
             printKey(network)
         elif inputFile != "" or address != "":
-            if network in reqUrl.keys():
-                if "etherscan" in reqUrl[network]:
-                    if proxy == "":
-                        print("In order to use the ETHSCAN API, you need to provide an proxy.")
-                        os._exit(0)
+            # Whether or not to use proxy modification is up to the user.
+            # if network in reqUrl.keys():
+            #     if "etherscan" in reqUrl[network]:
+            #         if proxy == "":
+            #             print("In order to use the ETHSCAN API, you need to provide an proxy.")
+            #             os._exit(0)
             getCode(inputFile, outputFolder, address, network)
+        elif txhash != "":
+            traceTransaction(txhash, proxy, network, outputFolder)
         else:
             print("Invalid command")
             os._exit(0)
+        if contractInfo!={}:
+            print("\nAddress => ContractName:")
+            for key in contractInfo.keys():
+                print(f"{key}\t{contractInfo[key]}")
+        if proxyContract!={}:
+            print("\nProxy => Implementation:")
+            for key in proxyContract.keys():
+                print(f"{key}\t{proxyContract[key]}")
         print('\nSuccess.')
 
     except Exception as e:
